from typing import Any

class Views:
    jenkins: Any = ...
    def __init__(self, jenkins: Any) -> None: ...
    async def get_all(self) -> dict: ...
    async def is_exists(self, name: str) -> bool: ...
    async def get_config(self, name: str) -> str: ...
