from aiojenkins.exceptions import JenkinsNotFoundError as JenkinsNotFoundError
from aiojenkins.utils import construct_job_config as construct_job_config
from typing import Any

class Jobs:
    jenkins: Any = ...
    def __init__(self, jenkins: Any) -> None: ...
    async def get_all(self) -> dict: ...
    async def get_info(self, name: str) -> dict: ...
    async def get_config(self, name: str) -> str: ...
    async def is_exists(self, name: str) -> bool: ...
    def construct_config(self, **kwargs: Any) -> str: ...
    async def create(self, name: str, config: str) -> None: ...
    async def delete(self, name: str) -> None: ...
    async def copy(self, name: str, new_name: str) -> None: ...
    async def rename(self, name: str, new_name: str) -> None: ...
    async def enable(self, name: str) -> None: ...
    async def disable(self, name: str) -> None: ...
