from typing import Any, Optional

class JenkinsError(Exception):
    message: Any = ...
    status: Any = ...
    def __init__(self, message: Optional[Any] = ..., status: Optional[Any] = ...) -> None: ...

class JenkinsNotFoundError(JenkinsError): ...
